module Order where

import qualified Linear
import qualified NewMaxPlus 
import Matrix
import Precedence
import ArgumentFiltering
{-
import Polynomial as P
import MaxPlus as MP
import Status
-}

data Order =
    Linear Linear.Algebra
  | Matrix Matrix.Algebra
  | NewMaxPlus NewMaxPlus.Algebra
  | LPO Precedence ArgumentFiltering
  | Lex [Order]
{-
  | LinearWPO P.Algebra Precedence Status
  | MaxPlusWPO MP.Algebra Precedence Status
  | LinearGWPO P.Algebra Status
  | MaxPlusGWPO MP.Algebra Status
  | LinearSPO P.Algebra Precedence Status
  | MaxPlusSPO MP.Algebra Precedence Status
  | Linear1LPO P.Algebra Precedence Status
  | MaxPlusLPO MP.Algebra Precedence
-}

indent :: Int -> String -> String
indent n text =
  unlines [ replicate n ' ' ++ line | line <- lines text ] 

instance Show Order where
  show (Linear a) = 
    "linear polynomial interpretations on natural numbers:\n\n" ++
    indent 2 (Linear.show_algebra a)
  show (Matrix a) =
    "matrix interpretations:\n\n" ++
    Matrix.show_algebra a
  show (NewMaxPlus a) =
    "max/plus interpretations on natural numbers:\n\n" ++
    indent 2 (NewMaxPlus.show_algebra a)
  show (LPO p af) =
    "lexicographic path order with precedence:\n\n" ++
    indent 2 (showPrecedence p) ++
    "  argument filter:\n\n" ++
    indent 4 (showArgumentFiltering af)
  show (Order.Lex os) =
    "lexicographic combination of reduction pairs:\n\n" ++
    indent 2 (concat [ show i ++ ". " ++ show o ++ "\n" 
                     | (i, o) <- zip [1::Int ..] os ])
{-
  show (LinearWPO a p st) =
    "weighted path orer with linear polynomial on natural numbers:\n" ++
    "\n" ++
    P.showAlgebra a ++
    "\n" ++
    showPrecedence p ++
    "\n" ++
    showStatus st
  show (MaxPlusWPO a p st) =
    "weighted path orer with max/plus on natural numbers:\n" ++
    "\n" ++
    MP.showAlgebra a ++
    "\n" ++
    showPrecedence p ++
    "\n" ++
    showStatus st
  show (LinearGWPO a st) =
    "generalized weighted path orer with linear polynomial on natural numbers:\n" ++
    "\n" ++
    P.showAlgebra a ++
    "\n" ++
    showStatus st
  show (MaxPlusGWPO a st) =
    "generalized weighted path orer with max/plus on natural numbers:\n" ++
    "\n" ++
    MP.showAlgebra a ++
    "\n" ++
    showStatus st
  show (LinearSPO a p st) =
    "semantic path orer with linear polynomial on natural numbers:\n" ++
    "\n" ++
    P.showAlgebra a ++
    "\n" ++
    showPrecedence p ++
    "\n" ++
    showStatus st
  show (MaxPlusSPO a p st) =
    "semantic path orer with max/plus on natural numbers:\n" ++
    "\n" ++
    MP.showAlgebra a ++
    "\n" ++
    showPrecedence p ++
    "\n" ++
    showStatus st
  show (Linear1LPO a p st) =
    "lexicographic comibnation of\n" ++
    "- linear polynomial on natural numbers and\n" ++
    P.showAlgebra a ++
    "- LPO with partial status\n" ++
    "\n" ++
    showPrecedence p ++
    "\n" ++
    showStatus st
  show (MaxPlusLPO a p) =
    "lexicographic comibnation of\n" ++
    "- max/plus on natural numbers and\n" ++
    MP.showAlgebra a ++
    "- LPO with partial status\n" ++
    "\n" ++
    showPrecedence p ++
    "\n" ++
    showEmptyStatus
-}