module Recursive where

import DP
import DG
import Proof

{-
import qualified LinearSPO
import qualified LinearWPO
import qualified LinearGWPO
import qualified MaxPlusSPO
import qualified MaxPlusWPO
import qualified MaxPlusGWPO
import qualified Linear1LPO
import qualified MaxPlusLPO
-}

type Criterion = Problem -> IO (Maybe Subproof)

apply :: [Criterion] -> Problem -> IO (Maybe Subproof)
apply [] _ = return Nothing
apply (criterion : criteria) problem = do
  result1 <- criterion problem
  case result1 of
    Nothing    -> apply criteria problem
    Just proof -> return (Just proof)


solve_problem :: Bool -> [Criterion] -> Problem -> IO (Maybe Proof)
solve_problem oneshot criteria problem = do
  result <- solve_sccs oneshot criteria problems
  case result of
    Nothing -> return Nothing
    Just proofs -> return (Just (Node problem (SCCs problem problems) proofs))
  where problems = if oneshot then [problem] else sccs problem

solve_problems :: Bool -> [Criterion] -> [Problem] -> IO (Maybe [Proof])
solve_problems _ _ [] = return (Just [])
solve_problems oneshot criteria (problem : problems) = do
  result1 <- solve_problem oneshot criteria problem
  case result1 of
    Nothing -> return Nothing
    Just proof -> do
      result2 <- solve_problems oneshot criteria problems
      case result2 of
        Nothing -> return Nothing
        Just proofs -> return (Just (proof : proofs))

solve_scc :: Bool -> [Criterion] -> Problem -> IO (Maybe Proof)
solve_scc oneshot criteria problem = do
  result1 <- apply criteria problem
  case result1 of
    Nothing -> return Nothing
    Just (criterion_proof, subproblems) -> do
      result2 <- solve_problems oneshot criteria subproblems
      case result2 of
        Nothing -> return Nothing
        Just subproofs -> 
          return (Just (Node problem criterion_proof subproofs))          

solve_sccs :: Bool -> [Criterion] -> [Problem] -> IO (Maybe [Proof])
solve_sccs _ _ [] = return (Just [])
solve_sccs oneshot criteria (problem : problems) = do
  result1 <- solve_scc oneshot criteria problem
  case result1 of
    Nothing    -> return Nothing
    Just proof -> do
      result2 <- solve_sccs oneshot criteria problems
      case result2 of
        Nothing -> return Nothing
        Just proofs -> return (Just (proof : proofs))

{-
wpol :: String -> Criterion
wpol tool problem = do
  result <- LinearWPO.solveOne tool problem
  case result of
    Nothing -> return Nothing
    Just (o, us, subproblem) -> 
      return (Just (ReductionPair problem o False us subproblem, [subproblem]))

wpom :: String -> Criterion
wpom tool problem = do
  result <- MaxPlusWPO.solveOne tool problem
  case result of
    Nothing -> return Nothing
    Just (o, us, subproblem) -> 
      return (Just (ReductionPair problem o False us subproblem, [subproblem]))

gwpol :: String -> Criterion
gwpol tool problem = do
  result <- LinearGWPO.solveOne tool problem
  case result of
    Nothing -> return Nothing
    Just (o, us, subproblem) -> 
      return (Just (ReductionPair problem o False us subproblem, [subproblem]))

gwpom :: String -> Criterion
gwpom tool problem = do
  result <- MaxPlusGWPO.solveOne tool problem
  case result of
    Nothing -> return Nothing
    Just (o, us, subproblem) -> 
      return (Just (ReductionPair problem o False us subproblem, [subproblem]))

spol :: String -> Criterion
spol tool problem = do
  result <- LinearSPO.solveOne tool problem
  case result of
    Nothing -> return Nothing
    Just (o, us, subproblem) -> 
      return (Just (ReductionPair problem o False us subproblem, [subproblem]))

spom :: String -> Criterion
spom tool problem = do
  result <- MaxPlusSPO.solveOne tool problem
  case result of
    Nothing -> return Nothing
    Just (o, us, subproblem) -> 
      return (Just (ReductionPair problem o False us subproblem, [subproblem]))


linear1lpo :: String -> Criterion
linear1lpo tool problem = do
  result <- Linear1LPO.solveOne tool problem
  case result of
    Nothing -> return Nothing
    Just (o, us, subproblem) -> 
      return (Just (ReductionPair problem o False us subproblem, [subproblem]))

maxpluslpo :: String -> Criterion
maxpluslpo tool problem = do
  result <- MaxPlusLPO.solveOne tool problem
  case result of
    Nothing -> return Nothing
    Just (o, us, subproblem) -> 
      return (Just (ReductionPair problem o False us subproblem, [subproblem]))

matrix :: String -> Matrix.BaseOrder -> Int -> Criterion
matrix tool base_order d problem = do
  result <- EncodingMatrix.solve_one tool base_order d problem
  case result of
    Nothing -> return Nothing
    Just (o, us, subproblem) -> 
      return (Just (ReductionPair o us, [subproblem]))
-}

